@@ -17,9 +17,9 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-__author__  = "Sebastian Heinlein <devel@glatzor.de>"
-__state__   = "stable"
-__version__ = "0.45"
+__author__ = "Sebastian Heinlein <devel@glatzor.de>"
+__state__ = "development"
+__version__ =  '1.0'
 
 __all__ = ("client", "console", "core", "debconf", "defer", "enums",
            "errors", "gtkwidgets", "loop", "policykit1", "progress",
