@@ -40,7 +40,7 @@ import sys
 
 from aptsources.sourceslist import SourceEntry
 import apt_pkg
-from gi.repository import GObject
+from gi.repository import GLib
 import dbus.glib
 import dbus.mainloop.glib
 dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
@@ -53,6 +53,8 @@ from . import errors
 ANSI_BOLD = chr(27) + "[1m"
 ANSI_RESET = chr(27) + "[0m"
 
+PY3K = sys.version_info.major > 2
+
 
 class ConsoleClient:
     """
@@ -83,7 +85,7 @@ class ConsoleClient:
         self._spin_cur = -1
         self._spin_stamp = time.time()
         self._transaction = None
-        self._loop = GObject.MainLoop()
+        self._loop = GLib.MainLoop()
 
     def add_repository(self, line="", sourcesfile=""):
         """Add repository to the sources list."""
@@ -96,15 +98,17 @@ class ConsoleClient:
 
     def add_vendor_key_from_file(self, path):
         """Install repository key file."""
-        self._client.add_vendor_key_from_file(path,
-                                            reply_handler=self._run_transaction,
-                                            error_handler=self._on_exception)
+        self._client.add_vendor_key_from_file(
+            path,
+            reply_handler=self._run_transaction,
+            error_handler=self._on_exception)
 
     def add_vendor_key_from_keyserver(self, keyid, keyserver):
         """Install repository key file."""
-        self._client.add_vendor_key_from_keyserver(keyid, keyserver,
-                                            reply_handler=self._run_transaction,
-                                            error_handler=self._on_exception)
+        self._client.add_vendor_key_from_keyserver(
+            keyid, keyserver,
+            reply_handler=self._run_transaction,
+            error_handler=self._on_exception)
 
     def remove_vendor_key(self, fingerprint):
         """Remove repository key."""
@@ -129,15 +133,16 @@ class ConsoleClient:
     def commit_packages(self, install, reinstall, remove, purge, upgrade,
                         downgrade):
         """Commit changes"""
-        self._client.commit_packages(install, reinstall, remove, purge, upgrade,
-                                     downgrade,
+        self._client.commit_packages(install, reinstall, remove, purge,
+                                     upgrade, downgrade,
                                      reply_handler=self._run_transaction,
                                      error_handler=self._on_exception)
 
     def fix_incomplete_install(self):
         """Fix incomplete installs"""
-        self._client.fix_incomplete_install(reply_handler=self._run_transaction,
-                                            error_handler=self._on_exception)
+        self._client.fix_incomplete_install(
+            reply_handler=self._run_transaction,
+            error_handler=self._on_exception)
 
     def fix_broken_depends(self):
         """Repair broken dependencies."""
@@ -173,7 +178,7 @@ class ConsoleClient:
     def _set_transaction(self, transaction):
         """Monitor the given transaction"""
         for handler in self._signals:
-            GObject.source_remove(handler)
+            GLib.source_remove(handler)
         self._transaction = transaction
         self._signals = []
         self._signals.append(transaction.connect("terminal-attached-changed",
@@ -196,18 +201,19 @@ class ConsoleClient:
         # Make sure to dettach the terminal
         self._detach()
         if self._show_progress:
-            sys.stderr.write("[+] 100% " + ANSI_BOLD +
-                             "%-*.*s" % (self._terminal_width - 9,
-                                         self._terminal_width - 9,
-                                         enums.get_exit_string_from_enum(enum))+
-                             ANSI_RESET + "\n")
+            output = "[+] 100%% %s %-*.*s%s\n" % (ANSI_BOLD,
+                     self._terminal_width - 9,
+                     self._terminal_width - 9,
+                     enums.get_exit_string_from_enum(enum),
+                     ANSI_RESET)
+            sys.stderr.write(output)
 
         if enum == enums.EXIT_FAILED:
             msg = "%s: %s\n%s\n\n%s" % (
-                   _("ERROR"),
-                   enums.get_error_string_from_enum(trans.error_code),
-                   enums.get_error_description_from_enum(trans.error_code),
-                   trans.error_details)
+                _("ERROR"),
+                enums.get_error_string_from_enum(trans.error_code),
+                enums.get_error_description_from_enum(trans.error_code),
+                trans.error_details)
             print(msg)
         self._loop.quit()
 
@@ -237,12 +243,12 @@ class ConsoleClient:
                              bytes_done, bytes_total, speed, eta):
         """Callback for the ProgressDetails signal of the transaction."""
         if bytes_total and speed:
-            self._progress_details = (_("Downloaded %sB of %sB at %sB/s") % \
+            self._progress_details = (_("Downloaded %sB of %sB at %sB/s") %
                                       (apt_pkg.size_to_str(bytes_done),
                                        apt_pkg.size_to_str(bytes_total),
                                        apt_pkg.size_to_str(speed)))
         elif bytes_total:
-            self._progress_details = (_("Downloaded %sB of %sB") % \
+            self._progress_details = (_("Downloaded %sB of %sB") %
                                       (apt_pkg.size_to_str(bytes_done),
                                        apt_pkg.size_to_str(bytes_total)))
         else:
@@ -300,7 +306,7 @@ class ConsoleClient:
     def _stop_custom_progress(self):
         """Stop the spinner which shows non trans status messages."""
         if self._progress_id is not None:
-            GObject.source_remove(self._progress_id)
+            GLib.source_remove(self._progress_id)
 
     def _clear_progress(self):
         """Clear progress information on stderr."""
@@ -310,8 +316,8 @@ class ConsoleClient:
 
     def _on_cancel_signal(self, signum, frame):
         """Callback for a cancel signal."""
-        if self._transaction and \
-           self._transaction.status != enums.STATUS_SETTING_UP:
+        if (self._transaction and
+                self._transaction.status != enums.STATUS_SETTING_UP):
             self._transaction.cancel()
         else:
             self._loop.quit()
@@ -324,29 +330,32 @@ class ConsoleClient:
     def _detach(self):
         """Dettach the controlling terminal to aptdaemon."""
         for wid in self._watchers:
-            GObject.source_remove(wid)
+            GLib.source_remove(wid)
         if self._old_tty_mode:
             tty.tcsetattr(pty.STDIN_FILENO, tty.TCSAFLUSH,
                           self._old_tty_mode)
 
     def _attach(self):
-        """Attach the controlling terminal to aptdaemon. Based on pty.spwan()"""
+        """Attach the controlling terminal to aptdaemon.
+        Based on pty.spwan()
+        """
         try:
             self._old_tty_mode = tty.tcgetattr(pty.STDIN_FILENO)
             tty.setraw(pty.STDIN_FILENO)
         except tty.error:    # This is the same as termios.error
             self._old_tty_mode = None
-        flags = GObject.IO_IN | GObject.IO_ERR | GObject.IO_HUP
-        wid = GObject.io_add_watch(pty.STDIN_FILENO, flags,
-                                   self._copy_io, self.master_fd)
-        self._watchers.append(wid)
-        wid = GObject.io_add_watch(self.master_fd, flags,
-                                   self._copy_io, pty.STDOUT_FILENO)
-        self._watchers.append(wid)
+        flags = GLib.IO_IN | GLib.IO_ERR | GLib.IO_HUP
+        self._watchers.append(
+            GLib.io_add_watch(pty.STDIN_FILENO,
+                              GLib.PRIORITY_HIGH_IDLE, flags,
+                              self._copy_io, self.master_fd))
+        self._watchers.append(
+            GLib.io_add_watch(self.master_fd, GLib.PRIORITY_HIGH_IDLE,
+                              flags, self._copy_io, pty.STDOUT_FILENO))
 
     def _copy_io(self, source, condition, target):
         """Callback to copy data between terminals."""
-        if condition == GObject.IO_IN:
+        if condition == GLib.IO_IN:
             data = os.read(source, 1024)
             if target:
                 os.write(target, data)
@@ -391,16 +400,17 @@ class ConsoleClient:
                                       enums.ROLE_REMOVE_VENDOR_KEY,
                                       enums.ROLE_FIX_INCOMPLETE_INSTALL]:
             #TRANSLATORS: status message
-            self._progress_id = \
-                    GObject.timeout_add(250, self._update_custom_progress,
-                                        _("Queuing"))
-            self._transaction.run(error_handler=self._on_exception,
-                     reply_handler=lambda: self._stop_custom_progress())
+            self._progress_id = GLib.timeout_add(250,
+                                                 self._update_custom_progress,
+                                                 _("Queuing"))
+            self._transaction.run(
+                error_handler=self._on_exception,
+                reply_handler=lambda: self._stop_custom_progress())
         else:
             #TRANSLATORS: status message
-            self._progress_id = \
-                    GObject.timeout_add(250, self._update_custom_progress,
-                                        _("Resolving dependencies"))
+            self._progress_id = GLib.timeout_add(250,
+                                                 self._update_custom_progress,
+                                                 _("Resolving dependencies"))
             self._transaction.simulate(reply_handler=self._show_changes,
                                        error_handler=self._on_exception)
 
@@ -419,8 +429,8 @@ class ConsoleClient:
                     output = "%s=%s" % (name, version)
                 else:
                     output = name
-                if len(line) + 1 + len(output) > self._terminal_width and \
-                   line != " ":
+                if (len(line) + 1 + len(output) > self._terminal_width and
+                        line != " "):
                     print(line)
                     line = " "
                 line += " %s" % output
@@ -428,10 +438,10 @@ class ConsoleClient:
                 print(line)
         self._stop_custom_progress()
         self._clear_progress()
-        installs, reinstalls, removals, purges, upgrades, downgrades  = \
-                self._transaction.packages
-        dep_installs, dep_reinstalls, dep_removals, dep_purges, dep_upgrades, \
-                dep_downgrades, dep_kepts = self._transaction.dependencies
+        (installs, reinstalls, removals, purges, upgrades,
+            downgrades) = self._transaction.packages
+        (dep_installs, dep_reinstalls, dep_removals, dep_purges, dep_upgrades,
+            dep_downgrades, dep_kepts) = self._transaction.dependencies
         installs.extend(dep_installs)
         upgrades.extend(dep_upgrades)
         removals.extend(purges)
@@ -442,197 +452,196 @@ class ConsoleClient:
         kepts = dep_kepts
         if installs:
             #TRANSLATORS: %s is the number of packages
-            print(ngettext("The following NEW package will be installed "
-                           "(%(count)s):",
-                           "The following NEW packages will be installed "
-                           "(%(count)s):",
-                           len(installs)) % {"count": len(installs)})
+            print((ngettext("The following NEW package will be installed "
+                            "(%(count)s):",
+                            "The following NEW packages will be installed "
+                            "(%(count)s):",
+                            len(installs)) % {"count": len(installs)}))
             show_packages(installs)
         if upgrades:
             #TRANSLATORS: %s is the number of packages
-            print(ngettext("The following package will be upgraded "
-                           "(%(count)s):",
-                           "The following packages will be upgraded "
-                           "(%(count)s):",
-                           len(upgrades)) % {"count": len(upgrades)})
+            print((ngettext("The following package will be upgraded "
+                            "(%(count)s):",
+                            "The following packages will be upgraded "
+                            "(%(count)s):",
+                            len(upgrades)) % {"count": len(upgrades)}))
             show_packages(upgrades)
         if removals:
             #TRANSLATORS: %s is the number of packages
-            print(ngettext("The following package will be REMOVED "
-                           "(%(count)s):",
-                           "The following packages will be REMOVED "
-                           "(%(count)s):",
-                           len(removals)) % {"count": len(removals)})
+            print((ngettext("The following package will be REMOVED "
+                            "(%(count)s):",
+                            "The following packages will be REMOVED "
+                            "(%(count)s):",
+                            len(removals)) % {"count": len(removals)}))
             #FIXME: mark purges
             show_packages(removals)
         if downgrades:
             #TRANSLATORS: %s is the number of packages
-            print(ngettext("The following package will be DOWNGRADED "
-                           "(%(count)s):",
-                           "The following packages will be DOWNGRADED "
-                           "(%(count)s):",
-                           len(downgrades)) % {"count": len(downgrades)})
+            print((ngettext("The following package will be DOWNGRADED "
+                            "(%(count)s):",
+                            "The following packages will be DOWNGRADED "
+                            "(%(count)s):",
+                            len(downgrades)) % {"count": len(downgrades)}))
             show_packages(downgrades)
         if reinstalls:
             #TRANSLATORS: %s is the number of packages
-            print(ngettext("The following package will be reinstalled "
-                           "(%(count)s):",
-                           "The following packages will be reinstalled "
-                           "(%(count)s):",
-                           len(reinstalls)) % {"count": len(reinstalls)})
+            print((ngettext("The following package will be reinstalled "
+                            "(%(count)s):",
+                            "The following packages will be reinstalled "
+                            "(%(count)s):",
+                            len(reinstalls)) % {"count": len(reinstalls)}))
             show_packages(reinstalls)
         if kepts:
-            print(ngettext("The following package has been kept back "
-                           "(%(count)s):",
-                           "The following packages have been kept back "
-                           "(%(count)s):",
-                           len(kepts)) % {"count": len(kepts)})
+            print((ngettext("The following package has been kept back "
+                            "(%(count)s):",
+                            "The following packages have been kept back "
+                            "(%(count)s):",
+                            len(kepts)) % {"count": len(kepts)}))
             show_packages(kepts)
 
         if self._transaction.download:
-            print(_("Need to get %sB of archives.") % \
-                    apt_pkg.size_to_str(self._transaction.download))
+            print(_("Need to get %sB of archives.") %
+                  apt_pkg.size_to_str(self._transaction.download))
         if self._transaction.space > 0:
             print(_("After this operation, %sB of additional disk space "
-                    "will be used.") % \
-                            apt_pkg.size_to_str(self._transaction.space))
+                    "will be used.") %
+                  apt_pkg.size_to_str(self._transaction.space))
         elif self._transaction.space < 0:
             print(_("After this operation, %sB of additional disk space "
-                    "will be freed.") % \
-                            apt_pkg.size_to_str(self._transaction.space))
+                    "will be freed.") %
+                  apt_pkg.size_to_str(self._transaction.space))
         if (not apt_pkg.config.find_b("APT::Get::Assume-Yes") and
             (self._transaction.space or self._transaction.download or
              installs or upgrades or downgrades or removals or kepts or
              reinstalls)):
             try:
-                cont = raw_input(_("Do you want to continue [Y/n]?"))
+                if PY3K:
+                    cont = input(_("Do you want to continue [Y/n]?"))
+                else:
+                    cont = raw_input(_("Do you want to continue [Y/n]?"))
             except EOFError:
                 cont = "n"
             #FIXME: Listen to changed dependencies!
-            if not re.match(locale.nl_langinfo(locale.YESEXPR), cont) and \
-               cont != "":
+            if (not re.match(locale.nl_langinfo(locale.YESEXPR), cont) and
+                    cont != ""):
                 msg = enums.get_exit_string_from_enum(enums.EXIT_CANCELLED)
                 self._update_custom_progress(msg, None, False)
                 self._loop.quit()
                 sys.exit(1)
         #TRANSLATORS: status message
-        self._progress_id = GObject.timeout_add(250,
-                                                self._update_custom_progress,
-                                                _("Queuing"))
-        self._transaction.run(error_handler=self._on_exception,
-                           reply_handler=lambda: self._stop_custom_progress())
+        self._progress_id = GLib.timeout_add(250,
+                                             self._update_custom_progress,
+                                             _("Queuing"))
+        self._transaction.run(
+            error_handler=self._on_exception,
+            reply_handler=lambda: self._stop_custom_progress())
 
 
 def main():
     """Run a command line client for aptdaemon"""
-    #FIXME: Workaround a bug in optparser which doesn't handle unicode/str
-    #       correctly, see http://bugs.python.org/issue4391
-    #       Shoudl be resolved by Python3
-    enc = locale.getpreferredencoding()
-
     epilog = _("To operate on more than one package put the package "
                "names in quotation marks:\naptdcon --install "
-               "\"foo bar\"").decode(enc)
+               "\"foo bar\"")
     parser = OptionParser(version=aptdaemon.__version__, epilog=epilog)
     parser.add_option("-c", "--refresh", default="",
                       action="store_true", dest="refresh",
-                      help=_("Refresh the cache").decode(enc))
+                      help=_("Refresh the cache"))
     parser.add_option("", "--fix-depends", default="",
                       action="store_true", dest="fix_depends",
                       help=_("Try to resolve broken dependencies. "
                              "Potentially dangerous operation since it could "
-                             "try to remove many packages.").decode(enc))
+                             "try to remove many packages."))
     parser.add_option("", "--fix-install", default="",
                       action="store_true", dest="fix_install",
                       help=_("Try to finish a previous incompleted "
-                             "installation").decode(enc))
+                             "installation"))
     parser.add_option("-i", "--install", default="",
                       action="store", type="string", dest="install",
-                      help=_("Install the given packages").decode(enc))
+                      help=_("Install the given packages"))
     parser.add_option("", "--reinstall", default="",
                       action="store", type="string", dest="reinstall",
-                      help=_("Reinstall the given packages").decode(enc))
+                      help=_("Reinstall the given packages"))
     parser.add_option("-r", "--remove", default="",
                       action="store", type="string", dest="remove",
-                      help=_("Remove the given packages").decode(enc))
+                      help=_("Remove the given packages"))
     parser.add_option("-p", "--purge", default="",
                       action="store", type="string", dest="purge",
                       help=_("Remove the given packages including "
-                             "configuration files").decode(enc))
+                             "configuration files"))
     parser.add_option("-u", "--upgrade", default="",
                       action="store", type="string", dest="upgrade",
-                      help=_("Install the given packages").decode(enc))
+                      help=_("Install the given packages"))
     parser.add_option("", "--downgrade", default="",
                       action="store", type="string", dest="downgrade",
-                      help=_("Downgrade the given packages").decode(enc))
+                      help=_("Downgrade the given packages"))
     parser.add_option("", "--upgrade-system",
                       action="store_true", dest="safe_upgrade",
                       help=_("Deprecated: Please use "
-                             "--safe-upgrade").decode(enc))
+                             "--safe-upgrade"))
     parser.add_option("", "--safe-upgrade",
                       action="store_true", dest="safe_upgrade",
-                      help=_("Upgrade the system in a safe way").decode(enc))
+                      help=_("Upgrade the system in a safe way"))
     parser.add_option("", "--full-upgrade",
                       action="store_true", dest="full_upgrade",
                       help=_("Upgrade the system, possibly installing and "
-                             "removing packages").decode(enc))
+                             "removing packages"))
     parser.add_option("", "--add-vendor-key", default="",
                       action="store", type="string", dest="add_vendor_key",
-                      help=_("Add the vendor to the trusted ones").decode(enc))
+                      help=_("Add the vendor to the trusted ones"))
     parser.add_option("", "--add-vendor-key-from-keyserver", default="",
-                      action="store", type="string", 
+                      action="store", type="string",
                       help=_("Add the vendor keyid (also needs "
-                             "--keyserver)").decode(enc))
+                             "--keyserver)"))
     parser.add_option("", "--keyserver", default="",
-                      action="store", type="string", 
+                      action="store", type="string",
                       help=_("Use the given keyserver for looking up "
-                             "keys").decode(enc))
-    parser.add_option("", "--add-repository", default="", 
+                             "keys"))
+    parser.add_option("", "--add-repository", default="",
                       action="store", type="string", dest="add_repository",
                       help=_("Add new repository from the given "
-                             "deb-line").decode(enc))
+                             "deb-line"))
     parser.add_option("", "--sources-file", action="store", default="",
                       type="string", dest="sources_file",
                       help=_("Specify an alternative sources.list.d file to "
-                             "which repositories should be added.").decode(enc))
+                             "which repositories should be added."))
     parser.add_option("", "--list-trusted-vendors", default="",
                       action="store_true", dest="list_trusted_vendor_keys",
-                      help=_("List trusted vendor keys").decode(enc))
+                      help=_("List trusted vendor keys"))
     parser.add_option("", "--remove-vendor-key", default="",
                       action="store", type="string", dest="remove_vendor_key",
                       help=_("Remove the trusted key of the given "
-                             "fingerprint").decode(enc))
+                             "fingerprint"))
     parser.add_option("", "--clean",
                       action="store_true", dest="clean",
-                      help=_("Remove downloaded package files").decode(enc))
+                      help=_("Remove downloaded package files"))
     parser.add_option("", "--reconfigure", default="",
                       action="store", type="string", dest="reconfigure",
                       help=_("Reconfigure installed packages. Optionally the "
                              "minimum priority of questions can be "
-                             "specified").decode(enc))
+                             "specified"))
     parser.add_option("", "--priority", default="default",
                       action="store", type="string", dest="priority",
                       help=_("The minimum debconf priority of question to "
-                             "be displayed").decode(enc))
+                             "be displayed"))
     parser.add_option("", "--hide-terminal",
                       action="store_true", dest="hide_terminal",
-                      help=_("Do not attach to the apt terminal").decode(enc))
+                      help=_("Do not attach to the apt terminal"))
     parser.add_option("", "--allow-unauthenticated",
                       action="store_true", dest="allow_unauthenticated",
                       default=False,
                       help=_("Allow packages from unauthenticated "
-                             "sources").decode(enc))
+                             "sources"))
     parser.add_option("-d", "--show-details",
                       action="store_true", dest="details",
                       help=_("Show additional information about the packages. "
-                             "Currently only the version number").decode(enc))
+                             "Currently only the version number"))
     (options, args) = parser.parse_args()
     con = ConsoleClient(show_terminal=not options.hide_terminal,
-                           allow_unauthenticated=options.allow_unauthenticated,
-                           details=options.details)
+                        allow_unauthenticated=options.allow_unauthenticated,
+                        details=options.details)
     #TRANSLATORS: status message
-    con._progress_id = GObject.timeout_add(250, con._update_custom_progress,
-                                           _("Waiting for authentication"))
+    con._progress_id = GLib.timeout_add(250, con._update_custom_progress,
+                                        _("Waiting for authentication"))
     if options.safe_upgrade:
         con.upgrade_system(True)
     elif options.full_upgrade:
@@ -649,8 +658,8 @@ def main():
         con.fix_broken_depends()
     elif options.install and options.install.endswith(".deb"):
         con.install_file(options.install)
-    elif options.install or options.reinstall or options.remove or \
-         options.purge or options.upgrade or options.downgrade:
+    elif (options.install or options.reinstall or options.remove or
+          options.purge or options.upgrade or options.downgrade):
         con.commit_packages(options.install.split(),
                             options.reinstall.split(),
                             options.remove.split(),
@@ -663,8 +672,9 @@ def main():
         #FIXME: Should detect if from stdin or file
         con.add_vendor_key_from_file(options.add_vendor_key)
     elif options.add_vendor_key_from_keyserver and options.keyserver:
-        con.add_vendor_key_from_keyserver(options.add_vendor_key_from_keyserver,
-                                          options.keyserver)
+        con.add_vendor_key_from_keyserver(
+            options.add_vendor_key_from_keyserver,
+            options.keyserver)
     elif options.remove_vendor_key:
         con.remove_vendor_key(options.remove_vendor_key)
     elif options.list_trusted_vendor_keys:
