@@ -17,11 +17,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-__author__  = "Sebastian Heinlein <devel@glatzor.de>"
-
-import gettext
-def _(msg):
-   return gettext.dgettext("aptdaemon", msg)
+__author__ = "Sebastian Heinlein <devel@glatzor.de>"
 
 __all__ = ("PKGS_INSTALL", "PKGS_REINSTALL", "PKGS_REMOVE", "PKGS_PURGE",
            "PKGS_UPGRADE", "PKGS_DOWNGRADE", "PKGS_KEEP",
@@ -38,6 +34,7 @@ __all__ = ("PKGS_INSTALL", "PKGS_REINSTALL", "PKGS_REMOVE", "PKGS_PURGE",
            "ERROR_SYSTEM_ALREADY_UPTODATE", "ERROR_NOT_SUPPORTED",
            "ERROR_LICENSE_KEY_INSTALL_FAILED",
            "ERROR_LICENSE_KEY_DOWNLOAD_FAILED",
+           "ERROR_AUTH_FAILED", "ERROR_NOT_AUTHORIZED",
            "ERROR_UNKNOWN",
            "STATUS_SETTING_UP", "STATUS_WAITING", "STATUS_WAITING_MEDIUM",
            "STATUS_WAITING_CONFIG_FILE_PROMPT", "STATUS_WAITING_LOCK",
@@ -68,6 +65,12 @@ __all__ = ("PKGS_INSTALL", "PKGS_REINSTALL", "PKGS_REMOVE", "PKGS_PURGE",
            "get_error_description_from_enum", "get_error_string_from_enum",
            "get_status_string_from_enum", "get_download_status_from_enum")
 
+import gettext
+
+
+def _(msg):
+    return gettext.dgettext("aptdaemon", msg)
+
 # PACKAGE GROUP INDEXES
 #: Index of the list of to be installed packages in the :attr:`dependencies`
 #: and :attr:`packages` property of :class:`~aptdaemon.client.AptTransaction`.
@@ -156,6 +159,10 @@ ERROR_LICENSE_KEY_DOWNLOAD_FAILED = "error-license-key-download-failed"
 ERROR_LICENSE_KEY_INSTALL_FAILED = "error-license-key-install-failed"
 #: The system is already up-to-date and don't needs any upgrades
 ERROR_SYSTEM_ALREADY_UPTODATE = "error-system-already-uptodate"
+#: The user isn't allowed to perform the action at all
+ERROR_NOT_AUTHORIZED = "error-not-authorized"
+#: The user could not be authorized (e.g. wrong password)
+ERROR_AUTH_FAILED = "error-auth-failed"
 #: An unknown error occured. In most cases these are programming ones.
 ERROR_UNKNOWN = "error-unknown"
 
@@ -304,19 +311,7 @@ _ICONS_STATUS = {
     STATUS_WAITING: 'aptdaemon-wait',
     STATUS_WAITING_LOCK: 'aptdaemon-wait',
     STATUS_WAITING_MEDIUM: 'aptdaemon-wait',
-    STATUS_WAITING_CONFIG_FILE_PROMPT:  'aptdaemon-wait',
-    }
-
-def get_status_icon_name_from_enum(enum):
-    """Get the icon name for a transaction status.
-
-    :param enum: The transaction status enum, e.g. :data:`STATUS_WAITING`.
-    :returns: The icon name string.
-    """
-    try:
-        return _ICONS_STATUS[enum]
-    except KeyError:
-        return "aptdaemon-working"
+    STATUS_WAITING_CONFIG_FILE_PROMPT:  'aptdaemon-wait'}
 
 _ICONS_ROLE = {
     ROLE_INSTALL_FILE: 'aptdaemon-add',
@@ -324,19 +319,7 @@ _ICONS_ROLE = {
     ROLE_UPDATE_CACHE: 'aptdaemon-update-cache',
     ROLE_REMOVE_PACKAGES: 'aptdaemon-delete',
     ROLE_UPGRADE_PACKAGES: 'aptdaemon-upgrade',
-    ROLE_UPGRADE_SYSTEM: 'system-software-update',
-    }
-
-def get_role_icon_name_from_enum(enum):
-    """Get an icon to represent the role of a transaction.
-
-    :param enum: The transaction role enum, e.g. :data:`ROLE_UPDATE_CACHE`.
-    :returns: The icon name string.
-    """
-    try:
-       return _ICONS_ROLE[enum]
-    except KeyError:
-       return "aptdaemon-working"
+    ROLE_UPGRADE_SYSTEM: 'system-software-update'}
 
 _ANIMATIONS_STATUS = {
     STATUS_CANCELLING: 'aptdaemon-action-cleaning-up',
@@ -348,103 +331,55 @@ _ANIMATIONS_STATUS = {
     STATUS_WAITING: 'aptdaemon-action-waiting',
     STATUS_WAITING_LOCK: 'aptdaemon-action-waiting',
     STATUS_WAITING_MEDIUM: 'aptdaemon-action-waiting',
-    STATUS_WAITING_CONFIG_FILE_PROMPT: 'aptdaemon-action-waiting',
-    }
-
-def get_status_animation_name_from_enum(enum):
-    """Get an animation to represent a transaction status.
-
-    :param enum: The transaction status enum, e.g. :data:`STATUS_WAITING`.
-    :returns: The animation name string.
-    """
-    try:
-        return _ANIMATIONS_STATUS[enum]
-    except KeyError:
-        return None
+    STATUS_WAITING_CONFIG_FILE_PROMPT: 'aptdaemon-action-waiting'}
 
 _PAST_ROLE = {
-    ROLE_INSTALL_FILE : _("Installed file"),
-    ROLE_INSTALL_PACKAGES : _("Installed packages"),
+    ROLE_INSTALL_FILE: _("Installed file"),
+    ROLE_INSTALL_PACKAGES: _("Installed packages"),
     ROLE_ADD_VENDOR_KEY_FILE: _("Added key from file"),
-    ROLE_UPDATE_CACHE : _("Updated cache"),
-    ROLE_PK_QUERY : _("Search done"),
-    ROLE_REMOVE_VENDOR_KEY : _("Removed trusted key"),
-    ROLE_REMOVE_PACKAGES : _("Removed packages"),
-    ROLE_UPGRADE_PACKAGES : _("Updated packages"),
-    ROLE_UPGRADE_SYSTEM : _("Upgraded system"),
-    ROLE_COMMIT_PACKAGES : _("Applied changes"),
-    ROLE_FIX_INCOMPLETE_INSTALL : _("Repaired incomplete installation"),
+    ROLE_UPDATE_CACHE: _("Updated cache"),
+    ROLE_PK_QUERY: _("Search done"),
+    ROLE_REMOVE_VENDOR_KEY: _("Removed trusted key"),
+    ROLE_REMOVE_PACKAGES: _("Removed packages"),
+    ROLE_UPGRADE_PACKAGES: _("Updated packages"),
+    ROLE_UPGRADE_SYSTEM: _("Upgraded system"),
+    ROLE_COMMIT_PACKAGES: _("Applied changes"),
+    ROLE_FIX_INCOMPLETE_INSTALL: _("Repaired incomplete installation"),
     ROLE_FIX_BROKEN_DEPENDS: _("Repaired broken dependencies"),
     ROLE_ADD_REPOSITORY: _("Added software source"),
     ROLE_ENABLE_DISTRO_COMP: _("Enabled component of the distribution"),
     ROLE_CLEAN: _("Removed downloaded package files"),
     ROLE_RECONFIGURE: _("Reconfigured installed packages"),
-    ROLE_UNSET: ""
-    }
-
-def get_role_localised_past_from_enum(enum):
-    """Get the description of a completed transaction action.
-
-    :param enum: The transaction role enum, e.g. :data:`ROLE_UPDATE_CACHE`.
-    :returns: The description string.
-    """
-    try:
-        return _PAST_ROLE[enum]
-    except KeyError:
-        return None
+    ROLE_UNSET: ""}
 
 _STRING_EXIT = {
-    EXIT_SUCCESS : _("Successful"),
-    EXIT_CANCELLED : _("Canceled"),
-    EXIT_FAILED : _("Failed")
-    }
-
-def get_exit_string_from_enum(enum):
-    """Get the description of a transaction exit status.
-
-    :param enum: The transaction exit status enum, e.g. :data:`EXIT_FAILED`.
-    :returns: The description string.
-    """
-    try:
-        return _STRING_EXIT[enum]
-    except:
-        return None
+    EXIT_SUCCESS: _("Successful"),
+    EXIT_CANCELLED: _("Canceled"),
+    EXIT_FAILED: _("Failed")}
 
 _PRESENT_ROLE = {
-    ROLE_INSTALL_FILE : _("Installing file"),
-    ROLE_INSTALL_PACKAGES : _("Installing packages"),
-    ROLE_ADD_VENDOR_KEY_FILE : _("Adding key from file"),
-    ROLE_UPDATE_CACHE : _("Updating cache"),
-    ROLE_REMOVE_VENDOR_KEY : _("Removing trusted key"),
-    ROLE_REMOVE_PACKAGES : _("Removing packages"),
-    ROLE_UPGRADE_PACKAGES : _("Updating packages"),
-    ROLE_UPGRADE_SYSTEM : _("Upgrading system"),
-    ROLE_COMMIT_PACKAGES : _("Applying changes"),
-    ROLE_FIX_INCOMPLETE_INSTALL : _("Repairing incomplete installation"),
+    ROLE_INSTALL_FILE: _("Installing file"),
+    ROLE_INSTALL_PACKAGES: _("Installing packages"),
+    ROLE_ADD_VENDOR_KEY_FILE: _("Adding key from file"),
+    ROLE_UPDATE_CACHE: _("Updating cache"),
+    ROLE_REMOVE_VENDOR_KEY: _("Removing trusted key"),
+    ROLE_REMOVE_PACKAGES: _("Removing packages"),
+    ROLE_UPGRADE_PACKAGES: _("Updating packages"),
+    ROLE_UPGRADE_SYSTEM: _("Upgrading system"),
+    ROLE_COMMIT_PACKAGES: _("Applying changes"),
+    ROLE_FIX_INCOMPLETE_INSTALL: _("Repairing incomplete installation"),
     ROLE_FIX_BROKEN_DEPENDS: _("Repairing installed software"),
     ROLE_ADD_REPOSITORY: _("Adding software source"),
     ROLE_ENABLE_DISTRO_COMP: _("Enabling component of the distribution"),
     ROLE_CLEAN: _("Removing downloaded package files"),
     ROLE_RECONFIGURE: _("Reconfiguring installed packages"),
-    ROLE_PK_QUERY : _("Searching"),
-    ROLE_UNSET : ""
-    }
-
-def get_role_localised_present_from_enum(enum):
-    """Get the description of a present transaction action.
-
-    :param enum: The transaction role enum, e.g. :data:`ROLE_UPDATE_CACHE`.
-    :returns: The description string.
-    """
-    try:
-        return _PRESENT_ROLE[enum]
-    except KeyError:
-        return None
+    ROLE_PK_QUERY: _("Searching"),
+    ROLE_UNSET: ""}
 
 _ERROR_ROLE = {
     ROLE_INSTALL_FILE: _("Installation of the package file failed"),
     ROLE_INSTALL_PACKAGES: _("Installation of software failed"),
-    ROLE_ADD_VENDOR_KEY_FILE: _("Adding the key to the list of trused "
+    ROLE_ADD_VENDOR_KEY_FILE: _("Adding the key to the list of trusted "
                                 "software vendors failed"),
     ROLE_UPDATE_CACHE: _("Refreshing the software list failed"),
     ROLE_REMOVE_VENDOR_KEY: _("Removing the vendor from the list of trusted "
@@ -453,73 +388,62 @@ _ERROR_ROLE = {
     ROLE_UPGRADE_PACKAGES: _("Updating software failed"),
     ROLE_UPGRADE_SYSTEM: _("Upgrading the system failed"),
     ROLE_COMMIT_PACKAGES: _("Applying software changes failed"),
-    ROLE_FIX_INCOMPLETE_INSTALL: _("Repairing incomplete installation failed"),
+    ROLE_FIX_INCOMPLETE_INSTALL: _("Repairing incomplete installation "
+                                   "failed"),
     ROLE_FIX_BROKEN_DEPENDS: _("Repairing broken dependencies failed"),
     ROLE_ADD_REPOSITORY: _("Adding software source failed"),
-    ROLE_ENABLE_DISTRO_COMP: _("Enabling component of the distribution failed"),
+    ROLE_ENABLE_DISTRO_COMP: _("Enabling component of the distribution "
+                               "failed"),
     ROLE_CLEAN: _("Removing downloaded package files failed"),
     ROLE_RECONFIGURE: _("Removing downloaded package files failed"),
     ROLE_PK_QUERY: _("Search failed"),
     ROLE_ADD_LICENSE_KEY: _("Adding license key"),
-    ROLE_UNSET: ""
-    }
-
-def get_role_error_from_enum(enum):
-    """Get the description of a failed transaction action.
-
-    :param enum: The transaction role enum, e.g. :data:`ROLE_UPDATE_CACHE`.
-    :returns: The description string.
-    """
-    try:
-        return _ERROR_ROLE[enum]
-    except KeyError:
-        return None
+    ROLE_UNSET: ""}
 
 _DESCS_ERROR = {
-    ERROR_PACKAGE_DOWNLOAD_FAILED : _("Check your Internet connection."),
-    ERROR_REPO_DOWNLOAD_FAILED : _("Check your Internet connection."),
-    ERROR_CACHE_BROKEN : _("Check if you are using third party "
-                           "repositories. If so disable them, since "
-                           "they are a common source of problems.\n"
-                           "Furthermore run the following command in a "
-                           "Terminal: apt-get install -f"),
-    ERROR_KEY_NOT_INSTALLED : _("The selected file may not be a GPG key file "
-                                "or it might be corrupt."),
-    ERROR_KEY_NOT_REMOVED : _("The selected key couldn't be removed. "
-                              "Check that you provided a valid fingerprint."),
-    ERROR_NO_LOCK : _("Check if you are currently running another "
-                      "software management tool, e.g. Synaptic or aptitude. "
-                      "Only one tool is allowed to make changes at a time."),
-    ERROR_NO_CACHE : _("This is a serious problem. Try again later. If this "
-                       "problem appears again, please report an error to the "
-                       "developers."),
-    ERROR_NO_PACKAGE : _("Check the spelling of the package name, and "
-                         "that the appropriate repository is enabled."),
-    ERROR_PACKAGE_UPTODATE : _("There isn't any need for an update."),
-    ERROR_PACKAGE_ALREADY_INSTALLED : _("There isn't any need for an "
-                                        "installation"),
-    ERROR_PACKAGE_NOT_INSTALLED : _("There isn't any need for a removal."),
-    ERROR_NOT_REMOVE_ESSENTIAL_PACKAGE : _("You requested to remove a package "
-                                           "which is an essential part of your "
-                                           "system."),
-    ERROR_DAEMON_DIED : _("The connection to the daemon was lost. Most likely "
-                          "the background daemon crashed."),
-    ERROR_PACKAGE_MANAGER_FAILED: _("The installation or removal of a software "
-                                    "package failed."),
+    ERROR_PACKAGE_DOWNLOAD_FAILED: _("Check your Internet connection."),
+    ERROR_REPO_DOWNLOAD_FAILED: _("Check your Internet connection."),
+    ERROR_CACHE_BROKEN: _("Check if you are using third party "
+                          "repositories. If so disable them, since "
+                          "they are a common source of problems.\n"
+                          "Furthermore run the following command in a "
+                          "Terminal: apt-get install -f"),
+    ERROR_KEY_NOT_INSTALLED: _("The selected file may not be a GPG key file "
+                               "or it might be corrupt."),
+    ERROR_KEY_NOT_REMOVED: _("The selected key couldn't be removed. "
+                             "Check that you provided a valid fingerprint."),
+    ERROR_NO_LOCK: _("Check if you are currently running another "
+                     "software management tool, e.g. Synaptic or aptitude. "
+                     "Only one tool is allowed to make changes at a time."),
+    ERROR_NO_CACHE: _("This is a serious problem. Try again later. If this "
+                      "problem appears again, please report an error to the "
+                      "developers."),
+    ERROR_NO_PACKAGE: _("Check the spelling of the package name, and "
+                        "that the appropriate repository is enabled."),
+    ERROR_PACKAGE_UPTODATE: _("There isn't any need for an update."),
+    ERROR_PACKAGE_ALREADY_INSTALLED: _("There isn't any need for an "
+                                       "installation"),
+    ERROR_PACKAGE_NOT_INSTALLED: _("There isn't any need for a removal."),
+    ERROR_NOT_REMOVE_ESSENTIAL_PACKAGE: _("You requested to remove a "
+                                          "package which is an essential "
+                                          "part of your system."),
+    ERROR_DAEMON_DIED: _("The connection to the daemon was lost. Most likely "
+                         "the background daemon crashed."),
+    ERROR_PACKAGE_MANAGER_FAILED: _("The installation or removal of a "
+                                    "software package failed."),
     ERROR_NOT_SUPPORTED: _("The requested feature is not supported."),
-    ERROR_UNKNOWN : _("There seems to be a programming error in aptdaemon, "
-                      "the software that allows you to install/remove "
-                      "software and to perform other package management "
-                      "related tasks."),
+    ERROR_UNKNOWN: _("There seems to be a programming error in aptdaemon, "
+                     "the software that allows you to install/remove "
+                     "software and to perform other package management "
+                     "related tasks."),
     ERROR_DEP_RESOLUTION_FAILED: _("This error could be caused by required "
                                    "additional software packages which are "
                                    "missing or not installable. Furthermore "
-                                   "there could be a conflict between software "
-                                   "packages which are not allowed to be "
-                                   "installed at the same time."),
-    ERROR_PACKAGE_UNAUTHENTICATED: _("The action would require the "
-                                     "installation of packages from not "
-                                     "authenticated sources."),
+                                   "there could be a conflict between "
+                                   "software packages which are not allowed "
+                                   "to be installed at the same time."),
+    ERROR_PACKAGE_UNAUTHENTICATED: _("This requires installing packages "
+                                     "from unauthenticated sources."),
     ERROR_INCOMPLETE_INSTALL: _("The installation could have failed because "
                                 "of an error in the corresponding software "
                                 "package or it was cancelled in an unfriendly "
@@ -531,10 +455,11 @@ _DESCS_ERROR = {
                                      "permissions."),
     ERROR_INVALID_PACKAGE_FILE: _("The installation of a package which "
                                   "violates the quality standards isn't "
-                                  "allowed. This could cause serious problems "
-                                  "on your computer. Please contact the person "
-                                  "or organisation who provided this package "
-                                  "file and include the details beneath."),
+                                  "allowed. This could cause serious "
+                                  "problems on your computer. Please contact "
+                                  "the person or organisation who provided "
+                                  "this package file and include the details "
+                                  "beneath."),
     ERROR_LICENSE_KEY_INSTALL_FAILED: _("The downloaded license key which is "
                                         "required to run this piece of "
                                         "software is not valid or could not "
@@ -544,38 +469,35 @@ _DESCS_ERROR = {
     ERROR_SYSTEM_ALREADY_UPTODATE: _("All available upgrades have already "
                                      "been installed."),
     ERROR_LICENSE_KEY_DOWNLOAD_FAILED: _("The license key which allows you to "
-                                         "use this piece of software could not be "
-                                         "downloaded. Please check your "
-                                         "network connection."),
-    }
-
-def get_error_description_from_enum(enum):
-    """Get a long description of an error.
-
-    :param enum: The transaction error enum, e.g. :data:`ERROR_NO_LOCK`.
-    :returns: The description string.
-    """
-    try:
-        return _DESCS_ERROR[enum]
-    except KeyError:
-        return None
+                                         "use this piece of software could "
+                                         "not be downloaded. Please check "
+                                         "your network connection."),
+    ERROR_NOT_AUTHORIZED: _("You don't have the required privileges to "
+                            "perform this action."),
+    ERROR_AUTH_FAILED: _("You either provided a wrong password or "
+                         "cancelled the authorization.\n"
+                         "Furthermore there could also be a technical reason "
+                         "for this error if you haven't seen a password "
+                         "dialog: your desktop environment doesn't provide a "
+                         "PolicyKit session agent.")}
 
 _STRINGS_ERROR = {
-    ERROR_PACKAGE_DOWNLOAD_FAILED : _("Failed to download package files"),
-    ERROR_REPO_DOWNLOAD_FAILED : _("Failed to download repository information"),
-    ERROR_DEP_RESOLUTION_FAILED : _("Package dependencies cannot be resolved"),
-    ERROR_CACHE_BROKEN : _("The package system is broken"),
-    ERROR_KEY_NOT_INSTALLED : _("Key was not installed"),
+    ERROR_PACKAGE_DOWNLOAD_FAILED: _("Failed to download package files"),
+    ERROR_REPO_DOWNLOAD_FAILED: _("Failed to download repository "
+                                  "information"),
+    ERROR_DEP_RESOLUTION_FAILED: _("Package dependencies cannot be resolved"),
+    ERROR_CACHE_BROKEN: _("The package system is broken"),
+    ERROR_KEY_NOT_INSTALLED: _("Key was not installed"),
     ERROR_KEY_NOT_REMOVED: _("Key was not removed"),
-    ERROR_NO_LOCK : _("Failed to lock the package manager"),
-    ERROR_NO_CACHE : _("Failed to load the package list"),
-    ERROR_NO_PACKAGE : _("Package does not exist"),
-    ERROR_PACKAGE_UPTODATE : _("Package is already up to date"),
-    ERROR_PACKAGE_ALREADY_INSTALLED : _("Package is already installed"),
-    ERROR_PACKAGE_NOT_INSTALLED : _("Package isn't installed"),
-    ERROR_NOT_REMOVE_ESSENTIAL_PACKAGE : _("Failed to remove essential "
-                                           "system package"),
-    ERROR_DAEMON_DIED : _("Task cannot be monitored or controlled"),
+    ERROR_NO_LOCK: _("Failed to lock the package manager"),
+    ERROR_NO_CACHE: _("Failed to load the package list"),
+    ERROR_NO_PACKAGE: _("Package does not exist"),
+    ERROR_PACKAGE_UPTODATE: _("Package is already up to date"),
+    ERROR_PACKAGE_ALREADY_INSTALLED: _("Package is already installed"),
+    ERROR_PACKAGE_NOT_INSTALLED: _("Package isn't installed"),
+    ERROR_NOT_REMOVE_ESSENTIAL_PACKAGE: _("Failed to remove essential "
+                                          "system package"),
+    ERROR_DAEMON_DIED: _("Task cannot be monitored or controlled"),
     ERROR_PACKAGE_MANAGER_FAILED: _("Package operation failed"),
     ERROR_PACKAGE_UNAUTHENTICATED: _("Requires installation of untrusted "
                                      "packages"),
@@ -586,62 +508,40 @@ _STRINGS_ERROR = {
     ERROR_LICENSE_KEY_DOWNLOAD_FAILED: _("Failed to download the license key"),
     ERROR_LICENSE_KEY_INSTALL_FAILED: _("Failed to install the license key"),
     ERROR_SYSTEM_ALREADY_UPTODATE: _("The system is already up to date"),
-    ERROR_UNKNOWN : _("An unhandlable error occured")
-    }
-
-def get_error_string_from_enum(enum):
-    """Get a short description of an error.
-
-    :param enum: The transaction error enum, e.g. :data:`ERROR_NO_LOCK`.
-    :returns: The description string.
-    """
-    try:
-        return _STRINGS_ERROR[enum]
-    except KeyError:
-        return None
+    ERROR_AUTH_FAILED: _("You could not be authorized"),
+    ERROR_NOT_AUTHORIZED: _("You are not allowed to perform this action"),
+    ERROR_UNKNOWN: _("An unhandlable error occured")}
 
 _STRINGS_STATUS = {
-    STATUS_SETTING_UP : _("Waiting for service to start"),
-    STATUS_QUERY : _("Searching"),
-    STATUS_WAITING : _("Waiting"),
-    STATUS_WAITING_MEDIUM : _("Waiting for required medium"),
-    STATUS_WAITING_LOCK : _("Waiting for other software managers to quit"),
-    STATUS_WAITING_CONFIG_FILE_PROMPT : _("Waiting for configuration file "
-                                          "prompt"),
-    STATUS_RUNNING : _("Running task"),
-    STATUS_DOWNLOADING : _("Downloading"),
-    STATUS_DOWNLOADING_REPO : _("Querying software sources"),
-    STATUS_CLEANING_UP : _("Cleaning up"),
-    STATUS_RESOLVING_DEP : _("Resolving dependencies"),
-    STATUS_COMMITTING : _("Applying changes"),
-    STATUS_FINISHED : _("Finished"),
-    STATUS_CANCELLING : _("Cancelling"),
-    STATUS_LOADING_CACHE : _("Loading software list"),
-    STATUS_AUTHENTICATING : _("Waiting for authentication"),
-    }
-
-def get_status_string_from_enum(enum):
-    """Get the description of a transaction status.
-
-    :param enum: The transaction status enum, e.g. :data:`STATUS_WAITING`.
-    :returns: The description string.
-    """
-    try:
-        return _STRINGS_STATUS[enum]
-    except KeyError:
-        return None
+    STATUS_SETTING_UP: _("Waiting for service to start"),
+    STATUS_QUERY: _("Searching"),
+    STATUS_WAITING: _("Waiting"),
+    STATUS_WAITING_MEDIUM: _("Waiting for required medium"),
+    STATUS_WAITING_LOCK: _("Waiting for other software managers to quit"),
+    STATUS_WAITING_CONFIG_FILE_PROMPT: _("Waiting for configuration file "
+                                         "prompt"),
+    STATUS_RUNNING: _("Running task"),
+    STATUS_DOWNLOADING: _("Downloading"),
+    STATUS_DOWNLOADING_REPO: _("Querying software sources"),
+    STATUS_CLEANING_UP: _("Cleaning up"),
+    STATUS_RESOLVING_DEP: _("Resolving dependencies"),
+    STATUS_COMMITTING: _("Applying changes"),
+    STATUS_FINISHED: _("Finished"),
+    STATUS_CANCELLING: _("Cancelling"),
+    STATUS_LOADING_CACHE: _("Loading software list"),
+    STATUS_AUTHENTICATING: _("Waiting for authentication")}
 
 STRINGS_PKG_STATUS = {
     #TRANSLATORS: %s is the name of a package
     PKG_INSTALLING: _("Installing %s"),
     #TRANSLATORS: %s is the name of a package
-    PKG_CONFIGURING: _( "Configuring %s"),
+    PKG_CONFIGURING: _("Configuring %s"),
     #TRANSLATORS: %s is the name of a package
-    PKG_REMOVING: _( "Removing %s"),
+    PKG_REMOVING: _("Removing %s"),
     #TRANSLATORS: %s is the name of a package
-    PKG_PURGING: _( "Completely removing %s"),
+    PKG_PURGING: _("Completely removing %s"),
     #TRANSLATORS: %s is the name of a package
-    PKG_PURGING: _( "Noting disappearance of %s"),
+    PKG_PURGING: _("Noting disappearance of %s"),
     #TRANSLATORS: %s is the name of a package
     PKG_RUNNING_TRIGGER: _("Running post-installation trigger %s"),
     #TRANSLATORS: %s is the name of a package
@@ -661,8 +561,136 @@ STRINGS_PKG_STATUS = {
     #TRANSLATORS: %s is the name of a package
     PKG_PURGED: _("Completely removed %s"),
     #TRANSLATORS: %s is the name of a package
-    PKG_REMOVED: _("Removed %s"),
-    }
+    PKG_REMOVED: _("Removed %s")}
+
+STRINGS_DOWNLOAD = {
+    DOWNLOAD_DONE: _("Done"),
+    DOWNLOAD_AUTH_ERROR: _("Authentication failed"),
+    DOWNLOAD_ERROR: _("Failed"),
+    DOWNLOAD_FETCHING: _("Fetching"),
+    DOWNLOAD_IDLE: _("Idle"),
+    DOWNLOAD_NETWORK_ERROR: _("Network isn't available")}
+
+
+def get_status_icon_name_from_enum(enum):
+    """Get the icon name for a transaction status.
+
+    :param enum: The transaction status enum, e.g. :data:`STATUS_WAITING`.
+    :returns: The icon name string.
+    """
+    try:
+        return _ICONS_STATUS[enum]
+    except KeyError:
+        return "aptdaemon-working"
+
+
+def get_role_icon_name_from_enum(enum):
+    """Get an icon to represent the role of a transaction.
+
+    :param enum: The transaction role enum, e.g. :data:`ROLE_UPDATE_CACHE`.
+    :returns: The icon name string.
+    """
+    try:
+        return _ICONS_ROLE[enum]
+    except KeyError:
+        return "aptdaemon-working"
+
+
+def get_status_animation_name_from_enum(enum):
+    """Get an animation to represent a transaction status.
+
+    :param enum: The transaction status enum, e.g. :data:`STATUS_WAITING`.
+    :returns: The animation name string.
+    """
+    try:
+        return _ANIMATIONS_STATUS[enum]
+    except KeyError:
+        return None
+
+
+def get_role_localised_past_from_enum(enum):
+    """Get the description of a completed transaction action.
+
+    :param enum: The transaction role enum, e.g. :data:`ROLE_UPDATE_CACHE`.
+    :returns: The description string.
+    """
+    try:
+        return _PAST_ROLE[enum]
+    except KeyError:
+        return None
+
+
+def get_exit_string_from_enum(enum):
+    """Get the description of a transaction exit status.
+
+    :param enum: The transaction exit status enum, e.g. :data:`EXIT_FAILED`.
+    :returns: The description string.
+    """
+    try:
+        return _STRING_EXIT[enum]
+    except:
+        return None
+
+
+def get_role_localised_present_from_enum(enum):
+    """Get the description of a present transaction action.
+
+    :param enum: The transaction role enum, e.g. :data:`ROLE_UPDATE_CACHE`.
+    :returns: The description string.
+    """
+    try:
+        return _PRESENT_ROLE[enum]
+    except KeyError:
+        return None
+
+
+def get_role_error_from_enum(enum):
+    """Get the description of a failed transaction action.
+
+    :param enum: The transaction role enum, e.g. :data:`ROLE_UPDATE_CACHE`.
+    :returns: The description string.
+    """
+    try:
+        return _ERROR_ROLE[enum]
+    except KeyError:
+        return None
+
+
+def get_error_description_from_enum(enum):
+    """Get a long description of an error.
+
+    :param enum: The transaction error enum, e.g. :data:`ERROR_NO_LOCK`.
+    :returns: The description string.
+    """
+    try:
+        return _DESCS_ERROR[enum]
+    except KeyError:
+        return None
+
+
+def get_error_string_from_enum(enum):
+    """Get a short description of an error.
+
+    :param enum: The transaction error enum, e.g. :data:`ERROR_NO_LOCK`.
+    :returns: The description string.
+    """
+    try:
+        return _STRINGS_ERROR[enum]
+    except KeyError:
+        return None
+
+
+def get_status_string_from_enum(enum):
+    """Get the description of a transaction status.
+
+    :param enum: The transaction status enum, e.g. :data:`STATUS_WAITING`.
+    :returns: The description string.
+    """
+    try:
+        return _STRINGS_STATUS[enum]
+    except KeyError:
+        return None
+
 
 def get_package_status_from_enum(enum):
     """Get the description of a package status.
@@ -673,16 +701,8 @@ def get_package_status_from_enum(enum):
     try:
         return STRINGS_PKG_STATUS[enum]
     except KeyError:
-        return _("Processing %s"),
+        return _("Processing %s")
 
-STRINGS_DOWNLOAD = {
-    DOWNLOAD_DONE: _("Done"),
-    DOWNLOAD_AUTH_ERROR: _("Authentication failed"),
-    DOWNLOAD_ERROR: _("Failed"),
-    DOWNLOAD_FETCHING: _("Fetching"),
-    DOWNLOAD_IDLE: _("Idle"),
-    DOWNLOAD_NETWORK_ERROR: _("Network isn't available"),
-    }
 
 def get_download_status_from_enum(enum):
     """Get the description of a download status.
